if not modules then modules = { } end modules ['t-annotation'] = {
    version   = 1.000,
    comment   = "Annotations",
    author    = "Wolfgang Schuster",
    copyright = "Wolfgang Schuster",
    license   = "GNU General Public License"
}

thirddata            = thirddata            or {}
thirddata.annotation = thirddata.annotation or {}

local annotation   = thirddata.annotation

local variables    = interfaces.variables

local v_yes        = variables.yes
local v_no         = variables.no
local v_auto       = variables.auto
local v_paragraph  = variables.paragraph
local v_text       = variables.text
local v_command    = variables.command
local v_vertical   = variables.vertical
local v_horizontal = variables.horizontal
local v_annotation = variables.annotation
local v_buffer     = variables.buffer

local texsprint    = tex.sprint
local ctxcatcodes  = tex.ctxcatcodes
local txtcatcodes  = tex.txtcatcodes

-- Collect the content of the environment

local data = { }

function annotation.erasedata(name)
    data[name] = nil
end

function annotation.getdata(name)
    local data = data[name]
    return data and data.content or ""
end

function annotation.printdata(name)
    local data     = data[name]
    local content  = data and data.content  or ""
    local catcodes = data and data.catcodes or ""
    if catcodes == txtcatcodes then
        texsprint(txtcatcodes,content:strip())
    else
        texsprint(ctxcatcodes,content:strip())
    end
end

function annotation.parameters(name)
    local data = data[name]
    local parameters = data and data.parameters or ""
    texsprint(ctxcatcodes,parameters)
end

function annotation.collectdata(name,parameters,content,begintag,endtag,catcodes)
    local oldcontent = annotation.getdata(name)
    local content    = content
    local parameters = parameters
    local catcodes   = catcodes
    local nesting    = false
    if oldcontent == "" then
        -- no nested environment
    else
        content = oldcontent .. endtag .. " " .. content
    end
    if select(2,string.gsub(content,begintag,begintag)) > select(2,string.gsub(content,endtag,endtag)) then
        nesting = true
    else
        nesting = false
    end
    data[name] = { parameters = parameters, content = content, catcodes = catcodes }
    commands.testcase(nesting)
end

function annotation.savedata(name,parameters,content,catcodes)
    data[name] = { parameters = parameters, content = content, catcodes = catcodes }
end

-- Write the content of the environment to a file

annotation.empty = true

function annotation.open()
    annotation.export = io.open(file.addsuffix(table.concat({tex.jobname,"annotation"},"-"),"txt"),"wb")
end

function annotation.write(title,content)
    local title, content = title:strip(), content
    if annotation.empty == true then
        annotation.open()
        annotation.empty = false
    else
        annotation.export:write("\n\n")
    end
    if title ~= "" then
        annotation.export:write(title)
        annotation.export:write("\n",string.rep("-",#title),"\n\n")
        content = content:gsub(" \\par ","\n\n")
        annotation.export:write(content)
    end
end

--~ function annotation.write(title,content)
--~     local title, content = title:strip(), content
--~     if annotation.empty == true then
--~         annotation.open()
--~         annotation.empty = false
--~     else
--~         annotation.export:write("\n")
--~     end
--~     annotation.export:write(title.." "..content)
--~ end
